package it4systems;

import java.awt.*;
import java.awt.geom.*;

import javax.swing.*;

public class MarsPanel extends JPanel {
	private MarsMap marsMap;
	private static final Color backgroundColor = Color.WHITE;
	private static final Color startColor = Color.BLACK;
	private static final Color targetColor = Color.RED;
	private static final Color stoneColor = Color.ORANGE;
	private int cellWidth = 10;
	private static final Font wonFont = new Font("SansSerif", Font.BOLD, 20);

	public void paintComponent(Graphics g) {
		g.setColor(backgroundColor);
		g.fillRect(0, 0, getWidth(), getHeight());
		if (marsMap != null) {
			int mapWidth = marsMap.getWidth();
			int mapHeight = marsMap.getHeight();
			for (int x = 0; x < mapWidth; x++) {
				for (int y = 0; y < mapHeight; y++) {
					int field = marsMap.getField(x, y);
					switch (field) {
						case MarsMap.START :
							g.setColor(startColor);
							break;
						case MarsMap.TARGET :
							g.setColor(targetColor);
							break;
						case MarsMap.STONE :
							g.setColor(stoneColor);
							break;
						default :
							g.setColor(backgroundColor);
					}
					if (field == MarsMap.TARGET && marsMap.isTargetReached(x, y) || field == MarsMap.START) {
						g.drawRect(x * cellWidth, y * cellWidth, cellWidth - 1, cellWidth - 1);
					} else {
						g.fillRect(x * cellWidth, y * cellWidth, cellWidth, cellWidth);
					}
				}
			}
			int x = marsMap.getRobotX() * cellWidth / 10;
			int y = marsMap.getRobotY() * cellWidth / 10;
			g.setColor(startColor);
			g.fillRect(x, y, cellWidth, cellWidth);
			if (marsMap.isWon()) {
				String result = "Won, Steps: " + marsMap.getSteps() + ", Fuel Usage: " + marsMap.getFuelUsage();
				Rectangle2D bounds = wonFont.getStringBounds(result, ((Graphics2D) g).getFontRenderContext());
				g.setColor(Color.BLACK);
				g.setFont(wonFont);
				g.drawString(
					result,
					(getWidth() - (int) bounds.getWidth()) / 2,
					(getHeight() - (int) bounds.getHeight()) / 2);
			}
		}
	}

	public Dimension getMaximumSize() {
		if (marsMap != null) {
			return new Dimension(marsMap.getWidth() * cellWidth, marsMap.getHeight() * cellWidth);
		} else {
			return new Dimension(100, 100);
		}
	}

	public Dimension getMinimumSize() {
		return getMaximumSize();
	}

	public Dimension getPreferredSize() {
		return getMaximumSize();
	}

	public void setMarsMap(MarsMap marsMap) {
		this.marsMap = marsMap;
		cellWidth = 800 / Math.max(marsMap.getWidth(), marsMap.getHeight());
		if (cellWidth > 10)
			cellWidth = 10;
		if (cellWidth < 1)
			cellWidth = 1;
		repaint();
	}
}
